@@ -8,7 +8,7 @@ use super::permission::*;
 use super::transaction::*;
 use crate::proto;
 
-use crate::errors::*;
+use crate::error::*;
 
 use stun::agent::*;
 use stun::attributes::*;
@@ -18,12 +18,12 @@ use stun::integrity::*;
 use stun::message::*;
 use stun::textattrs::*;
 
-use util::{Conn, Error};
+use util::Conn;
 
+use anyhow::Result;
 use std::io;
 use std::net::SocketAddr;
 use std::sync::Arc;
-
 use tokio::sync::{mpsc, Mutex};
 use tokio::time::{Duration, Instant};
 
@@ -43,13 +43,13 @@ pub trait RelayConnObserver {
     fn turn_server_addr(&self) -> String;
     fn username(&self) -> Username;
     fn realm(&self) -> Realm;
-    async fn write_to(&self, data: &[u8], to: &str) -> Result<usize, Error>;
+    async fn write_to(&self, data: &[u8], to: &str) -> Result<usize>;
     async fn perform_transaction(
         &mut self,
         msg: &Message,
         to: &str,
         ignore_result: bool,
-    ) -> Result<TransactionResult, Error>;
+    ) -> Result<TransactionResult>;
 }
 
 // RelayConnConfig is a set of configuration params use by NewUDPConn
@@ -109,7 +109,7 @@ impl<T: 'static + RelayConnObserver + Send + Sync> RelayConn<T> {
 
     // Close closes the connection.
     // Any blocked ReadFrom or write_to operations will be unblocked and return errors.
-    pub async fn close(&mut self) -> Result<(), Error> {
+    pub async fn close(&mut self) -> Result<()> {
         self.refresh_alloc_timer.stop();
         self.refresh_perms_timer.stop();
 
@@ -120,12 +120,12 @@ impl<T: 'static + RelayConnObserver + Send + Sync> RelayConn<T> {
 
 #[async_trait]
 impl<T: RelayConnObserver + Send + Sync> Conn for RelayConn<T> {
-    async fn connect(&self, _addr: SocketAddr) -> io::Result<()> {
-        Err(io::Error::new(io::ErrorKind::Other, "Not applicable"))
+    async fn connect(&self, _addr: SocketAddr) -> Result<()> {
+        Err(io::Error::new(io::ErrorKind::Other, "Not applicable").into())
     }
 
-    async fn recv(&self, _buf: &mut [u8]) -> io::Result<usize> {
-        Err(io::Error::new(io::ErrorKind::Other, "Not applicable"))
+    async fn recv(&self, _buf: &mut [u8]) -> Result<usize> {
+        Err(io::Error::new(io::ErrorKind::Other, "Not applicable").into())
     }
 
     // ReadFrom reads a packet from the connection,
@@ -138,7 +138,7 @@ impl<T: RelayConnObserver + Send + Sync> Conn for RelayConn<T> {
     // ReadFrom can be made to time out and return
     // an Error with Timeout() == true after a fixed time limit;
     // see SetDeadline and SetReadDeadline.
-    async fn recv_from(&self, p: &mut [u8]) -> io::Result<(usize, SocketAddr)> {
+    async fn recv_from(&self, p: &mut [u8]) -> Result<(usize, SocketAddr)> {
         let mut read_ch_rx = self.read_ch_rx.lock().await;
 
         if let Some(ib_data) = read_ch_rx.recv().await {
@@ -146,21 +146,23 @@ impl<T: RelayConnObserver + Send + Sync> Conn for RelayConn<T> {
             if p.len() < n {
                 return Err(io::Error::new(
                     io::ErrorKind::InvalidInput,
-                    ERR_SHORT_BUFFER.to_string(),
-                ));
+                    Error::ErrShortBuffer.to_string(),
+                )
+                .into());
             }
             p[..n].copy_from_slice(&ib_data.data);
             Ok((n, ib_data.from))
         } else {
             Err(io::Error::new(
                 io::ErrorKind::ConnectionAborted,
-                ERR_ALREADY_CLOSED.to_string(),
-            ))
+                Error::ErrAlreadyClosed.to_string(),
+            )
+            .into())
         }
     }
 
-    async fn send(&self, _buf: &[u8]) -> io::Result<usize> {
-        Err(io::Error::new(io::ErrorKind::Other, "Not applicable"))
+    async fn send(&self, _buf: &[u8]) -> Result<usize> {
+        Err(io::Error::new(io::ErrorKind::Other, "Not applicable").into())
     }
 
     // write_to writes a packet with payload p to addr.
@@ -168,16 +170,16 @@ impl<T: RelayConnObserver + Send + Sync> Conn for RelayConn<T> {
     // an Error with Timeout() == true after a fixed time limit;
     // see SetDeadline and SetWriteDeadline.
     // On packet-oriented connections, write timeouts are rare.
-    async fn send_to(&self, p: &[u8], addr: SocketAddr) -> io::Result<usize> {
+    async fn send_to(&self, p: &[u8], addr: SocketAddr) -> Result<usize> {
         let mut relay_conn = self.relay_conn.lock().await;
         match relay_conn.send_to(p, addr).await {
             Ok(n) => Ok(n),
-            Err(err) => Err(io::Error::new(io::ErrorKind::Other, err.to_string())),
+            Err(err) => Err(io::Error::new(io::ErrorKind::Other, err.to_string()).into()),
         }
     }
 
     // LocalAddr returns the local network address.
-    async fn local_addr(&self) -> io::Result<SocketAddr> {
+    async fn local_addr(&self) -> Result<SocketAddr> {
         Ok(self.relayed_addr)
     }
 }
@@ -201,7 +203,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
     // an Error with Timeout() == true after a fixed time limit;
     // see SetDeadline and SetWriteDeadline.
     // On packet-oriented connections, write timeouts are rare.
-    async fn send_to(&mut self, p: &[u8], addr: SocketAddr) -> Result<usize, Error> {
+    async fn send_to(&mut self, p: &[u8], addr: SocketAddr) -> Result<usize> {
         // check if we have a permission for the destination IP addr
         let mut perm = if let Some(perm) = self.perm_map.find(&addr) {
             *perm
@@ -215,7 +217,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         for _ in 0..MAX_RETRY_ATTEMPTS {
             result = self.create_perm(&mut perm, addr).await;
             if let Err(err) = &result {
-                if *err != *ERR_TRY_AGAIN {
+                if !Error::ErrTryAgain.equal(err) {
                     break;
                 }
             }
@@ -269,7 +271,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
                         {
                             let mut bm = binding_mgr.lock().await;
                             if let Err(err) = result {
-                                if err != *ERR_UNEXPECTED_RESPONSE {
+                                if !Error::ErrUnexpectedResponse.equal(&err) {
                                     bm.delete_by_addr(&bind_addr);
                                 } else if let Some(b) = bm.get_by_addr(&bind_addr) {
                                     b.set_state(BindingState::Failed);
@@ -326,7 +328,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
                     {
                         let mut bm = binding_mgr.lock().await;
                         if let Err(err) = result {
-                            if err != *ERR_UNEXPECTED_RESPONSE {
+                            if !Error::ErrUnexpectedResponse.equal(&err) {
                                 bm.delete_by_addr(&bind_addr);
                             } else if let Some(b) = bm.get_by_addr(&bind_addr) {
                                 b.set_state(BindingState::Failed);
@@ -356,7 +358,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
     // all the data transmission. This is done assuming that the request
     // will be mostly likely successful and we can tolerate some loss of
     // UDP packet (or reorder), inorder to minimize the latency in most cases.
-    async fn create_perm(&mut self, perm: &mut Permission, addr: SocketAddr) -> Result<(), Error> {
+    async fn create_perm(&mut self, perm: &mut Permission, addr: SocketAddr) -> Result<()> {
         if perm.state() == PermState::Idle {
             // punch a hole! (this would block a bit..)
             if let Err(err) = self.create_permissions(&[addr]).await {
@@ -368,7 +370,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         Ok(())
     }
 
-    async fn send_channel_data(&self, data: &[u8], ch_num: u16) -> Result<usize, Error> {
+    async fn send_channel_data(&self, data: &[u8], ch_num: u16) -> Result<usize> {
         let mut ch_data = proto::chandata::ChannelData {
             data: data.to_vec(),
             number: proto::channum::ChannelNumber(ch_num),
@@ -380,7 +382,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         obs.write_to(&ch_data.raw, &obs.turn_server_addr()).await
     }
 
-    async fn create_permissions(&mut self, addrs: &[SocketAddr]) -> Result<(), Error> {
+    async fn create_permissions(&mut self, addrs: &[SocketAddr]) -> Result<()> {
         let res = {
             let msg = {
                 let obs = self.obs.lock().await;
@@ -419,12 +421,12 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
             let mut code = ErrorCodeAttribute::default();
             let result = code.get_from(&res);
             if result.is_err() {
-                return Err(Error::new(format!("{}", res.typ)));
+                return Err(Error::new(format!("{}", res.typ)).into());
             } else if code.code == CODE_STALE_NONCE {
                 self.set_nonce_from_msg(&res);
-                return Err(ERR_TRY_AGAIN.to_owned());
+                return Err(Error::ErrTryAgain.into());
             } else {
-                return Err(Error::new(format!("{} (error {})", res.typ, code)));
+                return Err(Error::new(format!("{} (error {})", res.typ, code)).into());
             }
         }
 
@@ -444,16 +446,12 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
 
     // Close closes the connection.
     // Any blocked ReadFrom or write_to operations will be unblocked and return errors.
-    pub async fn close(&mut self) -> Result<(), Error> {
+    pub async fn close(&mut self) -> Result<()> {
         self.refresh_allocation(Duration::from_secs(0), true /* dontWait=true */)
             .await
     }
 
-    async fn refresh_allocation(
-        &mut self,
-        lifetime: Duration,
-        dont_wait: bool,
-    ) -> Result<(), Error> {
+    async fn refresh_allocation(&mut self, lifetime: Duration, dont_wait: bool) -> Result<()> {
         let res = {
             let mut obs = self.obs.lock().await;
 
@@ -489,10 +487,10 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
             let mut code = ErrorCodeAttribute::default();
             let result = code.get_from(&res);
             if result.is_err() {
-                return Err(Error::new(format!("{}", res.typ)));
+                return Err(Error::new(format!("{}", res.typ)).into());
             } else if code.code == CODE_STALE_NONCE {
                 self.set_nonce_from_msg(&res);
-                return Err(ERR_TRY_AGAIN.to_owned());
+                return Err(Error::ErrTryAgain.into());
             } else {
                 return Ok(());
             }
@@ -507,7 +505,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         Ok(())
     }
 
-    async fn refresh_permissions(&mut self) -> Result<(), Error> {
+    async fn refresh_permissions(&mut self) -> Result<()> {
         let addrs = self.perm_map.addrs();
         if addrs.is_empty() {
             log::debug!("no permission to refresh");
@@ -515,7 +513,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         }
 
         if let Err(err) = self.create_permissions(&addrs).await {
-            if err != *ERR_TRY_AGAIN {
+            if !Error::ErrTryAgain.equal(&err) {
                 log::error!("fail to refresh permissions: {}", err);
             }
             return Err(err);
@@ -531,7 +529,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         bind_number: u16,
         nonce: Nonce,
         integrity: MessageIntegrity,
-    ) -> Result<(), Error> {
+    ) -> Result<()> {
         let (msg, turn_server_addr) = {
             let obs = rc_obs.lock().await;
 
@@ -563,7 +561,7 @@ impl<T: RelayConnObserver + Send + Sync> RelayConnInternal<T> {
         let res = tr_res.msg;
 
         if res.typ != MessageType::new(METHOD_CHANNEL_BIND, CLASS_SUCCESS_RESPONSE) {
-            return Err(ERR_UNEXPECTED_RESPONSE.to_owned());
+            return Err(Error::ErrUnexpectedResponse.into());
         }
 
         log::debug!("channel binding successful: {} {}", bind_addr, bind_number);
@@ -586,7 +584,7 @@ impl<T: RelayConnObserver + Send + Sync> PeriodicTimerTimeoutHandler for RelayCo
                 for _ in 0..MAX_RETRY_ATTEMPTS {
                     result = self.refresh_allocation(lifetime, false).await;
                     if let Err(err) = &result {
-                        if *err != *ERR_TRY_AGAIN {
+                        if !Error::ErrTryAgain.equal(err) {
                             break;
                         }
                     }
@@ -600,7 +598,7 @@ impl<T: RelayConnObserver + Send + Sync> PeriodicTimerTimeoutHandler for RelayCo
                 for _ in 0..MAX_RETRY_ATTEMPTS {
                     result = self.refresh_permissions().await;
                     if let Err(err) = &result {
-                        if *err != *ERR_TRY_AGAIN {
+                        if !Error::ErrTryAgain.equal(err) {
                             break;
                         }
                     }
